class NoWeb3Exception(Exception):
    pass

class DisabledException(Exception):
    pass

class DeploymentInfoNotFound(Exception):
    pass

class ContractMetadataNotFound(Exception):
    pass

class NameServiceNotSet(Exception):
    pass
